program NetSend;
{$APPTYPE CONSOLE}
uses SysUtils, Windows;

const
  maxMsgLength = 900;

function SendMsg(Dest, Source, Msg: string) : Longint; overload;

type
  TNetMessageBufferSendFunction = function(servername, msgname, fromname: PWideChar; Buf: PWideChar; buflen: Cardinal) : Longint; stdcall;
var
  NetMessageBufferSend: TNetMessageBufferSendFunction;
  SourceWideChar: PWideChar;
  DestWideChar: PWideChar;
  MessagetextWideChar: PWideChar;
  Handle: THandle;

begin { SendMsg }
  Handle := LoadLibrary('NETAPI32.DLL');
  if Handle=0 then
  begin
    Result := GetLastError;
    Exit;
  end; { Handle=0 }
  @NetMessageBufferSend := GetProcAddress(Handle, 'NetMessageBufferSend');
  if @NetMessageBufferSend=nil then
  begin
    Result := GetLastError;
    Exit;
  end; { @NetMessageBufferSend=nil }
  MessagetextWideChar := nil;
  SourceWideChar := nil;
  DestWideChar := nil;

  try
    GetMem(MessagetextWideChar, Length(Msg)*SizeOf(WideChar)+
           1);
    GetMem(DestWideChar, 20*SizeOf(WideChar)+1);
    StringToWideChar(Msg, MessagetextWideChar, Length(Msg)*SizeOf(WideChar)+
                     1);
    StringToWideChar(Dest, DestWideChar, 20*SizeOf(WideChar)+
                     1);
    if Source='' then begin
      Result := NetMessageBufferSend(nil, DestWideChar, nil, MessagetextWideChar, Length(Msg)*SizeOf(WideChar)+1);
    end
    else begin
      GetMem(SourceWideChar, 20*SizeOf(WideChar)+1);
      StringToWideChar(Source, SourceWideChar, 20*SizeOf(WideChar)+ 1);
      Result := NetMessageBufferSend(nil, DestWideChar, SourceWideChar, MessagetextWideChar, Length(Msg)*SizeOf(WideChar)+1);
      FreeMem(SourceWideChar);
    end; { not (Source='') }

  finally
    FreeMem(MessagetextWideChar);
    FreeLibrary(Handle);
  end; { try }
end; { SendMsg }

function SendMsg(Dest, Msg: string) : Longint; overload;
begin { SendMsg }
  Result := SendMsg(Dest, '', Msg);
end; { SendMsg }

function SendMsg(Msg: string) : Longint; overload;
begin { SendMsg }
  Result := SendMsg('', '', Msg);
end; { SendMsg }

Procedure Usage;

begin
  Writeln('The usage of this command is:');
  Writeln;
  Writeln;
  Writeln('Netsend <MSGTARGET> <MSGFILE>');
  Writeln;
  Writeln('MSGTARGET is a valid message alias (computer name or user name, for example');
  Writeln('MSGFILE is the name of a text file, with a mximum of 1000 characters');
end;

function ParamCheck(var sTarget: string; var sMsgFile: string) : Boolean;

// Require two parameters: message target and message filename

begin
  sMsgFile := '';
  sTarget := '';
  Result := (paramcount = 2);
  if not Result then begin
    usage;
    exit;
  end;
  sTarget :=  paramStr(1);
  sMsgFile := paramStr(2);
end;

function GetMsg(const sfn: string; var sMsg : string) : Boolean;

var
  s, sLine : string;
  F : TextFile;

begin
  Result := FileExists(sfn);
  if (not Result) then begin
    Writeln('Specified message file [' + sfn + '] does not exist');
    exit;
  end;
  try
    AssignFile(F, sfn);
    try
      Reset(F);
      While not EOF(F) do begin
        Readln(F, sLine);
        if (sMsg <> '') then sMsg := sMsg + Chr(13) + sLine
        else smsg := sMsg + sLine;
      end;
    finally
      CloseFile(F);
    end;
  except
    on e:Exception do begin
      Writeln('Error reading message file [' + sfn + ']');
      Writeln('Error message was: ' + e.message);
      Result := False;
      exit;
    end;
  end;
  if (Length(sMsg) > maxMsgLength) then begin
    Result := false;
    Str(maxmsglength, s);
    Writeln('Message file too long (limit is ' + s + ' characters)');
    exit;
  end;
  Result := True;
end;

var
  sMsgFile: string;
  sTarget : string;
  sMsg : string;
  retval : LongInt;
  s : string;

begin
  if not paramcheck(sTarget, sMsgFile) then begin
    halt(1);
  end
  else begin
    if not GetMsg(sMsgFile, sMsg) then begin
      halt(1);
    end
    else begin
      retval := SendMsg(sTarget, sMsg);
      if (retval = 0) then begin
        Writeln('Message sent successfully');
        halt(0);
      end
      else begin
        Str(retval, s);
        Writeln('Error sending message; error [' + s + ']');
        halt(1);
      end;
    end;
  end;
end.  // Insert user code here
